Block Storage Broker Charm
-------------------------

This charm intends to handle all credentials and interactions with cloud block
storage services, such as nova, so that other charms don't need to have
visibility to those credentials or commands necessary to create and attach
volumes to juju units. Volume consumers need only request volume size and provide their instance-id.
This charm runs as a standalone unit but could easily share resources with a
more computationally intensive service.

Block storage broker provides one interface.
    - volume-request: The volume consumer, your charm or the storage 
      subordinate charm, will specify via relation-set the instance-id and an
      optional size. A volume of that size will be created, if necessary, and
      attached to the instance-id provided. When the requested volume is
      attached, the block-storage-broker charm will publish the
      block-device-path via the relation data to announce the device path that
      is available for mount in the instance-id.



Usage
-----
Currently this charm is intended for use with the storage subordinate charm
which will allow the principal units to request and automatically mount volumes.
The command set below will relate your charm to storage and storage to the block-storage-broker and allocate an 11 Gig nova volume for each postgresql unit.
    $ cat >block-storage-bundle.cfg <<END
common:
    services:
        postgresql:
            branch: lp:~fcorrea/charms/precise/postgresql/delegate-blockstorage-to-storage-subordinate
            constraints: mem=2048
            options:
                extra-packages: python-apt postgresql-contrib postgresql-9.1-debversion
                max_connections: 500
        storage:
            branch: lp:~davidpbritton/charms/precise/storage/trunk
            options:
                volume_size: 11
                provider: blockstoragebroker
                root: /srv/data
        block-storage-broker:
            branch: lp:~chad.smith/charms/precise/block-storage-broker/trunk
            options:
                key: <YOUR_OS_USERNAME>
                endpoint: <YOUR_OS_AUTH_URL>
                region: <YOUR_OS_REGION>
                secret: <YOUR_OS_PASSWORD>
                tenant: <YOUR_OS_TENANT_NAME>
doit:
    inherits: common
    series: precise
    relations:
        - ["postgresql:data", "storage:data"]
        - ["storage", "block-storage-broker"]
END

    $ juju-deployer -c block-storage-bundle.cfg doit

Sample Data Relation
--------------------

The Block Storage Broker charm is designed to make your life easier when
requesting volume attachments to your instances. As a charm author,
Implementing the data relation is very straightforward.

Sample Metadata:

requires:
    block-storage-broker:
        interface: volume-request

Sample Joined Hook:

    #!/bin/bash

    # Grab you instance id
    METADATA_URL=http://169.254.169.254/openstack/2012-08-10/meta_data.json
    INSTANCE_ID=`curl $METADATA_URL | jsonpipe | grep uuid | awk '{ print $2 }'`

    relation-set size=11
    relation-set instance-id=$INSTANCE_ID
    relation-set volume-label=$JUJU_UNIT_NAME

Sample Changed Hook:

    #!/bin/bash
    device_path=`relation-get block-device-path`
    if [ -z "$device_path" ] ; then
        juju-log "wait for related service to start"
        exit 0
    fi
    service my_service stop
    mount $device_path /somewhere
    change_my_service_to_use_mount /somewhere
    service my_service start


TODO
----

 * Block Storage Broker to support Amazon storage solutions
 * Investigate Block Storage Broker discovery of instance-id if no instance-id provided
 * Allow config option to automatically destroy a volume upon relation departed/broken
